% This MATLAB script reproduces the results in the article: 
% De Winter, J. C. F., & Nordhoff, S. (2022). Acceptance of conditionally automated cars: Just one factor? Transportation Research Interdisciplinary Perspectives. https://doi.org/10.1016/j.trip.2022.100645

% Tested using MATLAB R2021b

clear variables;close all;clc

F=0; % if 0, then load mat-file. if 1, then load Excel file
if F==1
    X=xlsread('L3Pilot_Global_UA_Survey_1st_Phase_Without_WTP.xlsx',2,'A2:GI18632'); %#ok<*XLSRD> % Downloaded from https://zenodo.org/record/5255950#.YrdN-XbP1mM
    [~,T]=xlsread('L3Pilot_Global_UA_Survey_1st_Phase_Without_WTP.xlsx',2,'C2:F18632');
    duration=NaN(length(T),1);
    warning off all
    for i=1:length(T) % compute the questionnaire completion time for all participants
        try
            starttime=datenum(T{i,1},'dd/mm/yy')+datenum(T{i,2});
            endtime=datenum(T{i,3},'dd/mm/yy')+datenum(T{i,4});
        catch error
            starttime=datenum(T{i,1},'dd.mm.yyyy')+datenum(T{i,2});
            endtime=datenum(T{i,3},'dd.mm.yyyy')+datenum(T{i,4});
        end
        try
            duration(i)=24*3600*(endtime-starttime);
        catch error
            duration(i)=NaN;
        end
    end
    warning on all
    save('X.mat','X','duration')
else
     load X % 18631 x 191 matrix
end
%%
Xr=[X(:,[60 72:97 155:159]) sum(X(:,98:101),2,'omitnan')]; % retain a data matrix (18590 x 33)
        % 60: f14
        % 72-74: f15_1 to f15_3
        % 75-97: f16_1 to f16_23
        % 155-159: f20_1 to f20_5
        % 98-101: f17 (a, b, c, or d)

for i=-1+[1 2 4 5 6 7 9 10] % add 8 items 
    Xr=[Xr sum(X(:,[107+i 119+i 131+i 143+i]),2,'omitnan')]; %#ok<AGROW> 
end

% 107 119 131 143: f19_1 (a, b, c, or d)
% 108 120 132 144: f19_2 (a, b, c, or d)
% 110 122 134 146: f19_4 (a, b, c, or d)
% 111 123 135 147: f19_5 (a, b, c, or d)
% 112 124 136 148: f19_6 (a, b, c, or d)
% 113 125 137 149: f19_7 (a, b, c, or d)
% 115 127 139 151: f19_9 (a, b, c, or d)
% 116 128 140 152: f19_10 (a, b, c, or d)

% Xr is now 18590 x 44

Xr(Xr==6 | Xr==7)=NaN; % set 'I prefer not to respond' to NaN. Also set 'I don't take such trips' (f20) to NaN.

invalidpp=sum(isnan(Xr'))>20;
Xr(invalidpp,:)=[]; % remove 41 participants with many missing responses
X(invalidpp,:)=[]; % remove 41 participants with many missing responses
duration(invalidpp)=[]; % remove 41 participants with many missing responses

disp(['Missing data percentage = ' num2str(100*sum(sum(isnan(Xr)))./(size(Xr,1)*size(Xr,2))) ' %'])

Xi=knnimpute(Xr); % impute missing data
Xiz=zscore(Xi); % create z-transformation
[lding,GAF,eigr,~,varexplained]=pca(Xiz); % principal component analysis
GAF=zscore(GAF(:,1)); % compute general acceptance score (vector of length 18590)
lding=lding(:,1).*sqrt(eigr(1)); % pca outputs the loadings normalized (sum of squared loadings equals 1). Here we un-normalize the loadings
%% Figure 4. GAF loadings for fast respondents and slow respondents. Each marker represents a questionnaire item. The diagonal line is the line of unity. The questionnaire completion time data was available for 14,809 of 18,590 respondents. The median questionnaire completion time was 13.75 min.
opengl hardware
ldingS=NaN(length(lding),2);
[NPS,varexplainedS]=deal(NaN(2,1)); 
for i=1:2 % loop over two groups (fast, slow)
    if i==1
        ppselected=find(round(duration)<600);
    elseif i==2
        ppselected=find(round(duration)>1200);
    end
    disp(['Number of participants in group = ' num2str(length(ppselected))])
    NPS(i)=length(ppselected);
    [ldings,~,eigS,~,varexplaineds]=pca(zscore(Xiz(ppselected,:))); % principal component analysis
    ldingS(:,i)=ldings(:,1).*sqrt(eigS(1));
    varexplainedS(i)=varexplaineds(1);
end
figure
for i=1:size(Xiz,2)
    scatter1=scatter(ldingS(i,1),ldingS(i,2),600,'markerfacecolor','b','markeredgecolor','b');hold on;grid on
    scatter1.MarkerFaceAlpha = .3;
end
plot([0.5 0.85],[0.5 0.85],'k')
axis equal
set(gca,'xlim',[0.5 0.85])
set(gca,'ylim',[0.5 0.85])
xlabel(['GAF loading for fast respondents, < 10 min (\it{n}\rm = ' num2str(NPS(1)) ')'])
ylabel(['GAF loading for slow respondents, > 20 min (\it{n}\rm = ' num2str(NPS(2)) ')'])
h=findobj('FontName','Helvetica');set(h,'FontSize',20,'Fontname','Arial')

disp(['Variance explained for fast respondents = ' num2str(varexplainedS(1)) ' %'])
disp(['Variance explained for slow respondents = ' num2str(varexplainedS(2)) ' %'])
disp(['Number of participants with available task completion time = ' num2str(sum(~isnan(duration)))])
disp(['Median questionnaire completion time = ' num2str(median(duration/60,'omitnan')) ' min'])
%% Figure 1. Eigenvalues of the correlation matrix, sorted in descending order ("scree plot"). Also shown are the percentages of variance explained (being proportional to the eigenvalue) for the first five factors. It can be seen that one dominant factor emerged. 
opengl hardware
figure;plot(eigr,'k-o','Linewidth',3)
ylabel('Eigenvalue')
xlabel('Eigenvalue number')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',28)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01],'xtick',[1 5:5:100]);
set(gca,'xlim',[.5 length(eigr)+.5])
for i=1:6
    text(i+0.2,eigr(i)+0.4,[sprintf('%0.1f',varexplained(i)) '%'],'color','b','fontsize',14);
end
grid on
%%
AgeVector=18:69; % vector of ages

[MS,Nm,Nf,No,N]=deal(NaN(max(AgeVector),1));
CI=NaN(max(AgeVector),2);
[MSC,NC]=deal(NaN(max(AgeVector),17));

for i=AgeVector % loop over all ages
    temp=find(X(:,7)==i); % find indexes of age i
    MS(i)=mean(GAF(temp)); % general acceptance factor for that age
    [~,~,CI(i,:)]=ttest(GAF(temp)); % 95% confidence interval for that age
    Nm(i)=sum(X(temp,8)==1); % number of males per age
    Nf(i)=sum(X(temp,8)==2); % number of females per age
    No(i)=sum(X(temp,8)==3); % number of other per age
    for i2=1:17 % loop over countries
        temp=find(X(:,7)==i & X(:,1)==i2); % find indexes of age and country
        MSC(i,i2)=mean(GAF(temp)); % mean general acceptance score for age and country
        NC(i,i2)=sum(~isnan(GAF(temp))); % sample size per country and age
    end
end
%% Figure 2. Mean general acceptance score for each age separately. Also shown is a parabolic fit and 95% confidence intervals calculated for each age separately. The top of the figure lists the number of males (in blue), females (in pink), and participants with the gender “other” (in grey) for the age groups 18–20, 21–25, 26–30, 31–35, 36–40, 41–45, 46–50, 51–55, 56–60, 61–65, and 66–69.
opengl hardware
figure;
plot(AgeVector,MS(AgeVector),'ko-','markerfacecolor','k','Linewidth',3,'markersize',14);hold on
for i=AgeVector
    plot([i i],[CI(i,1) CI(i,2)],'b-s')
end
PL=polyval(polyfit(AgeVector,MS(AgeVector),2),18:0.1:69); % 2nd degree polynomial fit between age and GAF score

[GAFc,Age,Gender,NN]=deal(NaN(17,1));
[PLC,PLCV]=deal(NaN(17,3));
for i2=1:17 % loop over 17 countries
    GAFc(i2)=mean(GAF(X(:,1)==i2),'omitnan'); % Mean GAF score
    Age(i2)=mean(X(X(:,1)==i2,7)); % Mean age
    Gender(i2)=100*mean(X(X(:,1)==i2,8)==2,'omitnan'); % percentage females
    NN(i2)=sum(~isnan(GAF(X(:,1)==i2))); % Sample size
    validindexes=find(~isnan(MSC(AgeVector,i2)));
    PLC(i2,:)=polyfit(AgeVector(validindexes),MSC(AgeVector(validindexes),i2),2); % a, b, and c coefficients
    PLCV(i2,:)=polyval(polyfit(AgeVector(validindexes),MSC(AgeVector(validindexes),i2),2),[18 35 69]); % GAF score at ages 18, 35, and 69
end

V=[19 22.5:5:62.5 67.0]; % x-value for plotting the numeric value
L=[18 21:5:61 66]; % lower age range
H=[20:5:65 69]; % upper age range

plot(18:0.1:69,PL,'r','Linewidth',5);hold on % plot the second-degree polynomial
for i=1:length(V)
    text(V(i),0.545,num2str(sum(Nm(L(i):H(i)))),'horizontalalignment','center','color','b') % number of males
    text(V(i),0.490,num2str(sum(Nf(L(i):H(i)))),'horizontalalignment','center','color','m') % number of females
    text(V(i),0.435,num2str(sum(No(L(i):H(i)))),'horizontalalignment','center','color',[.5 .5 .5]) % number of other
end
grid on
xlabel('Age (years)');
ylabel('General acceptance (\it{z}\rm-score)');
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.1],'xlim',[18 69],'xtick',[18 20:5:65 69])
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',28)
disp(['Mean general acceptance score for age 69 = ' num2str(MS(69))])

%% Figure 3. Distribution of general acceptance scores for four age groups. The horizontal axis shows the general acceptance scores in bins of 0.2 and the vertical axis shows the number of participants, normalized so that the total equals 1.
x1 =  GAF(X(:,7)>=18 & X(:,7)<=20);
x2 =  GAF(X(:,7)>=21 & X(:,7)<=40);
x3 =  GAF(X(:,7)>=41 & X(:,7)<=60);
x4 =  GAF(X(:,7)>=61 & X(:,7)<=69);

V=-4:0.2:2;
x1h=histc(x1,V);x2h=histc(x2,V);x3h=histc(x3,V);x4h=histc(x4,V); %#ok<HISTC> % counts per GAF score interval 
x1h=x1h./sum(x1h);x2h=x2h./sum(x2h);x3h=x3h./sum(x3h);x4h=x4h./sum(x4h); % normalize so that the total per age curve equals 1
figure;
plot(V+mean(diff(V))/2,x1h,'-o','color',[.8 0 0],'Linewidth',4,'markersize',10,'markerfacecolor',[.8 0 0]);hold on
plot(V+mean(diff(V))/2,x2h,'-d','color',[0 .8 0],'Linewidth',4,'markersize',10,'markerfacecolor',[0 .8 0])
plot(V+mean(diff(V))/2,x3h,'-s','color',[0 0 1],'Linewidth',4,'markersize',10,'markerfacecolor',[0 0 1])
plot(V+mean(diff(V))/2,x4h,'-^','color',[.6 .6 .6],'Linewidth',4,'markersize',10,'markerfacecolor',[.6 .6 .6])
legend('18–20 years','21–40 years','41–60 years','61–69 years')
xlabel('General acceptance (\it{z}\rm-score)');
grid on
ylabel('Probability')
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'xlim',[-4 2])
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',28)
%%
disp('Correlation GAF and age')
[c1,p1]=corr(X(:,7),GAF,'type','pearson');
disp([c1 p1])
%%
disp('Table 1, loadings in descending order')
[~,lding_sorted]=sort(lding,'descend');  % lding(lding_sorted) gives the indexes of loadings dorted in descending order, from high to low. For example, lding_sorted(1) = 14 means that the highest-loading item is Item 14
[~,lding_sorted2]=sort(lding_sorted); % Ranking of loadings, where 1 = highest loading and 41 = lowest loading. For example, lding_sorted2(26) = 2 means that Item 26 has the second-highest loading
disp([mean(Xi(:,lding_sorted))' lding(lding_sorted)]) % Display the mean item score (first column) and the GAF loading (second column)
disp('Mean, min, max loading')
disp([mean(lding) min(lding) max(lding)])
disp('Second-degree polynomial fit coefficients, all participants (a, b, c)')
disp(polyfit(AgeVector,MS(AgeVector),2))

disp('Table 2 - GAF-related effects for respondents from different countries')
disp([NN Gender Age GAFc PLCV PLC])
%%
mlfa_loadings=factoran(Xiz,6,'rotate','promax'); % Loadings of maximum likelihood factor analysis (MLFA). Extract 6 factors

uw_scores = NaN(length(Xiz),6);
for i=1:size(mlfa_loadings,2) % loop over the 6 factors
    disp(['Factor ' num2str(i) ' Items:'])
    disp(sort(lding_sorted2(mlfa_loadings(:,i)>.5))'); % display item numbers (corresponding to Table 1) loading higher than 0.5
    uw_scores(:,i)=mean(Xiz(:,mlfa_loadings(:,i)>.5),2); % mean score per participant across the items loading higher than 0.5
end

disp('Correlation matrix of unit-weight scores')
disp(round(corr(uw_scores),2))

y=zscore(X(:,7)); % Age. Standardize so that the standard deviation equals 1
x=zscore(uw_scores(:,[1 2 3])); % First three factor scores. Standardize so that the standard deviation equals 1
st=regstats(y,x); % perform linear regression analysis
disp('Regression analysis results for predicting age (beta, tstat, p-value, zero-order correlation). First row is the intercept')
disp([[st.beta st.tstat.t st.tstat.pval(1:end)] [NaN;corr(x,y)]])
